@@ -5,7 +5,8 @@ use crate::Vec;
 use ark_crypto_primitives::CRHGadget;
 use ark_ff::PrimeField;
 use ark_r1cs_std::{
-	alloc::AllocVar, eq::EqGadget, prelude::*, select::CondSelectGadget, ToBytesGadget,
+	alloc::AllocVar, eq::EqGadget, fields::fp::FpVar, prelude::*, select::CondSelectGadget,
+	ToBytesGadget,
 };
 use ark_relations::r1cs::{Namespace, SynthesisError};
 use ark_std::{borrow::Borrow, rc::Rc};
@@ -142,6 +143,15 @@ where
 		root: &NodeVar<F, P, HG, LHG>,
 		leaf: L,
 	) -> Result<Boolean<F>, SynthesisError> {
+		let computed_root = self.root_hash(&leaf)?;
+
+		root.is_eq(&computed_root)
+	}
+
+	pub fn root_hash<L: ToBytesGadget<F>>(
+		&self,
+		leaf: &L,
+	) -> Result<NodeVar<F, P, HG, LHG>, SynthesisError> {
 		assert_eq!(self.path.len(), P::HEIGHT as usize);
 		// Check that the hash of the given leaf matches the leaf hash in the membership
 		// proof.
@@ -175,7 +185,43 @@ where
 			)?;
 		}
 
-		root.is_eq(&previous_hash)
+		Ok(previous_hash)
+	}
+
+	pub fn get_index<L: ToBytesGadget<F>>(
+		&self,
+		root: &NodeVar<F, P, HG, LHG>,
+		leaf: L,
+	) -> Result<FpVar<F>, SynthesisError> {
+		// First, check if the provided leaf is on the path
+		let isonpath = self.check_membership(root, &leaf);
+		isonpath.unwrap().enforce_equal(&Boolean::TRUE)?;
+
+		let mut index = FpVar::<F>::zero();
+		let mut twopower = FpVar::<F>::one();
+		let mut rightvalue: FpVar<F>;
+
+		//Compute the hash of the provided leaf
+		let leaf_hash = hash_leaf_gadget::<F, P, HG, LHG, L>(self.leaf_params.borrow(), &leaf)?;
+
+		// Check levels between leaf level and root.
+		let mut previous_hash = leaf_hash;
+		for &(ref left_hash, ref right_hash) in self.path.iter() {
+			// Check if the previous_hash is for a left node.
+			let previous_is_left = previous_hash.is_eq(&left_hash)?;
+
+			rightvalue = index.clone() + twopower.clone();
+			index = FpVar::<F>::conditionally_select(&previous_is_left, &index, &rightvalue)?;
+			twopower = twopower.clone() + twopower.clone();
+
+			previous_hash = hash_inner_node_gadget::<F, P, HG, LHG>(
+				self.inner_params.borrow(),
+				left_hash,
+				right_hash,
+			)?;
+		}
+
+		Ok(index)
 	}
 }
 
@@ -257,40 +303,25 @@ where
 	}
 }
 
-#[cfg(feature = "default_poseidon")]
 #[cfg(test)]
 mod test {
 	use super::{NodeVar, PathVar};
 	use crate::{
 		ark_std::UniformRand,
-		merkle_tree::{Config, SparseMerkleTree},
-		poseidon::{
-			constraints::CRHGadget as PoseidonCRHGadget, sbox::PoseidonSbox, PoseidonParameters,
-			Rounds, CRH as PoseidonCRH,
-		},
-		utils::{get_mds_poseidon_bls381_x5_3, get_rounds_poseidon_bls381_x5_3},
+		merkle_tree::{Config, Path, SparseMerkleTree},
+		poseidon::{constraints::CRHGadget as PoseidonCRHGadget, CRH as PoseidonCRH},
 	};
+
 	use ark_bls12_381::Fq;
 	use ark_r1cs_std::{alloc::AllocVar, fields::fp::FpVar, R1CSVar};
 	use ark_relations::r1cs::ConstraintSystem;
 	use ark_std::{rc::Rc, test_rng};
-
-	use crate::merkle_tree::Path;
+	use arkworks_utils::utils::common::{setup_params_x5_3, Curve};
 
 	type FieldVar = FpVar<Fq>;
 
-	#[derive(Default, Clone)]
-	struct PoseidonRounds3;
-
-	impl Rounds for PoseidonRounds3 {
-		const FULL_ROUNDS: usize = 8;
-		const PARTIAL_ROUNDS: usize = 57;
-		const SBOX: PoseidonSbox = PoseidonSbox::Exponentiation(5);
-		const WIDTH: usize = 3;
-	}
-
-	type SMTCRH = PoseidonCRH<Fq, PoseidonRounds3>;
-	type SMTCRHGadget = PoseidonCRHGadget<Fq, PoseidonRounds3>;
+	type SMTCRH = PoseidonCRH<Fq>;
+	type SMTCRHGadget = PoseidonCRHGadget<Fq>;
 
 	#[derive(Clone, Debug, Eq, PartialEq)]
 	struct SMTConfig;
@@ -307,9 +338,10 @@ mod test {
 	#[test]
 	fn should_verify_path() {
 		let rng = &mut test_rng();
-		let rounds3 = get_rounds_poseidon_bls381_x5_3::<Fq>();
-		let mds3 = get_mds_poseidon_bls381_x5_3::<Fq>();
-		let params3 = PoseidonParameters::<Fq>::new(rounds3, mds3);
+		let curve = Curve::Bls381;
+
+		let params3 = setup_params_x5_3(curve);
+
 		let inner_params = Rc::new(params3);
 		let leaf_params = inner_params.clone();
 
@@ -333,13 +365,46 @@ mod test {
 		assert!(res.value().unwrap());
 	}
 
+	#[test]
+	fn should_verify_index() {
+		let rng = &mut test_rng();
+		let curve = Curve::Bls381;
+
+		let params3 = setup_params_x5_3(curve);
+
+		let inner_params = Rc::new(params3);
+		let leaf_params = inner_params.clone();
+		let index = 2;
+		let cs = ConstraintSystem::<Fq>::new_ref();
+
+		let leaves = vec![Fq::rand(rng), Fq::rand(rng), Fq::rand(rng)];
+		let smt = SMT::new_sequential(inner_params, leaf_params, &leaves).unwrap();
+		let root = smt.root();
+		let path = smt.generate_membership_proof(index);
+
+		let path_var =
+			PathVar::<_, _, _, _, { SMTConfig::HEIGHT as usize }>::new_witness(cs.clone(), || {
+				Ok(path)
+			})
+			.unwrap();
+		let root_var = SMTNode::new_witness(cs.clone(), || Ok(root)).unwrap();
+		let leaf_var = FieldVar::new_witness(cs.clone(), || Ok(leaves[index as usize])).unwrap();
+
+		let res = path_var.get_index(&root_var, &leaf_var).unwrap();
+		let desired_res = Fq::from(index);
+
+		assert!(res.cs().is_satisfied().unwrap());
+		assert_eq!(res.value().unwrap(), desired_res);
+	}
+
 	#[should_panic(expected = "Expected a Vec of length 2 but it was 3")]
 	#[test]
 	fn should_fail_path_creation_with_invalid_size() {
 		let rng = &mut test_rng();
-		let rounds3 = get_rounds_poseidon_bls381_x5_3::<Fq>();
-		let mds3 = get_mds_poseidon_bls381_x5_3::<Fq>();
-		let params3 = PoseidonParameters::<Fq>::new(rounds3, mds3);
+		let curve = Curve::Bls381;
+
+		let params3 = setup_params_x5_3(curve);
+
 		let inner_params = Rc::new(params3);
 		let leaf_params = inner_params.clone();
 
@@ -371,9 +436,10 @@ mod test {
 	#[test]
 	fn should_fail_membership_with_invalid_size() {
 		let rng = &mut test_rng();
-		let rounds3 = get_rounds_poseidon_bls381_x5_3::<Fq>();
-		let mds3 = get_mds_poseidon_bls381_x5_3::<Fq>();
-		let params3 = PoseidonParameters::<Fq>::new(rounds3, mds3);
+		let curve = Curve::Bls381;
+
+		let params3 = setup_params_x5_3(curve);
+
 		let inner_params = Rc::new(params3);
 		let leaf_params = inner_params.clone();
 
