@@ -1,5 +1,4 @@
 use super::{MiMCParameters, Rounds, CRH};
-use crate::utils::to_field_var_elements;
 use ark_crypto_primitives::crh::constraints::{CRHGadget as CRHGadgetTrait, TwoToOneCRHGadget};
 use ark_ff::PrimeField;
 use ark_r1cs_std::{
@@ -10,6 +9,7 @@ use ark_r1cs_std::{
 };
 use ark_relations::r1cs::{Namespace, SynthesisError};
 use ark_std::{marker::PhantomData, vec::Vec};
+use arkworks_utils::utils::to_field_var_elements;
 use core::borrow::Borrow;
 
 #[derive(Clone)]
@@ -189,7 +189,6 @@ impl<F: PrimeField> AllocVar<MiMCParameters<F>, F> for MiMCParametersVar<F> {
 	}
 }
 
-#[cfg(all(feature = "mimc_220_ed_on_bn254"))]
 #[cfg(test)]
 mod test {
 	#![allow(non_camel_case_types)]
@@ -197,7 +196,7 @@ mod test {
 	use super::*;
 	use ark_crypto_primitives::crh::CRH as CRHTrait;
 	use ark_ed_on_bn254::Fq;
-	use ark_ff::{field_new, to_bytes, Zero};
+	use ark_ff::{to_bytes, Zero};
 	use ark_relations::r1cs::ConstraintSystem;
 
 	#[derive(Default, Clone)]
@@ -231,7 +230,7 @@ mod test {
 			MiMCRounds220_3::ROUNDS,
 			MiMCRounds220_3::WIDTH,
 			MiMCRounds220_3::WIDTH,
-			crate::utils::get_rounds_mimc_220(),
+			arkworks_utils::utils::get_rounds_mimc_220(),
 		);
 
 		let params_var =
@@ -270,7 +269,7 @@ mod test {
 			MiMCRounds220_2::ROUNDS,
 			MiMCRounds220_2::WIDTH,
 			MiMCRounds220_2::WIDTH,
-			crate::utils::get_rounds_mimc_220(),
+			arkworks_utils::utils::get_rounds_mimc_220(),
 		);
 
 		let params_var =
