@@ -1,4 +1,3 @@
-use super::Set;
 use ark_crypto_primitives::Error;
 use ark_ff::{bytes::ToBytes, fields::PrimeField, to_bytes};
 use ark_std::marker::PhantomData;
@@ -31,10 +30,8 @@ pub struct SetMembership<F: PrimeField, const M: usize> {
 	field: PhantomData<F>,
 }
 
-impl<F: PrimeField, const M: usize> Set<F, M> for SetMembership<F, M> {
-	type Private = Private<F, M>;
-
-	fn generate_secrets<T: ToBytes>(target: &T, set: &[F; M]) -> Result<Self::Private, Error> {
+impl<F: PrimeField, const M: usize> SetMembership<F, M> {
+	pub fn generate_secrets<T: ToBytes>(target: &T, set: &[F; M]) -> Result<Private<F, M>, Error> {
 		let target_bytes = to_bytes![target]?;
 		let t = F::read(target_bytes.as_slice())?;
 		let mut diffs = [F::default(); M];
@@ -44,7 +41,7 @@ impl<F: PrimeField, const M: usize> Set<F, M> for SetMembership<F, M> {
 		Ok(Private::new(&diffs))
 	}
 
-	fn check<T: ToBytes>(target: &T, set: &[F; M], s: &Self::Private) -> Result<bool, Error> {
+	pub fn check<T: ToBytes>(target: &T, set: &[F; M], s: &Private<F, M>) -> Result<bool, Error> {
 		let target_bytes = to_bytes![target]?;
 		let target = F::read(target_bytes.as_slice())?;
 		let mut product = target.clone();
@@ -58,11 +55,32 @@ impl<F: PrimeField, const M: usize> Set<F, M> for SetMembership<F, M> {
 
 		Ok(product == F::zero())
 	}
+
+	pub fn check_is_enabled<T: ToBytes>(
+		target: &T,
+		set: &[F; M],
+		s: &Private<F, M>,
+		enabled: &F,
+	) -> Result<bool, Error> {
+		let target_bytes = to_bytes![target]?;
+		let target = F::read(target_bytes.as_slice())?;
+		let mut product = target.clone();
+		let z = F::zero();
+		for (diff, real) in s.diffs.iter().zip(set.iter()) {
+			if z != ((*real - *diff - target) * enabled) {
+				return Ok(false);
+			}
+			product *= diff;
+		}
+		let result = product * enabled;
+		Ok(result == F::zero())
+	}
 }
 
 #[cfg(test)]
 mod test {
 	use super::*;
+	use crate::ark_std::Zero;
 	use ark_bls12_381::Fq;
 	use ark_ff::UniformRand;
 	use ark_std::test_rng;
@@ -82,6 +100,27 @@ mod test {
 		assert!(is_member);
 	}
 
+	#[test]
+	fn should_test_membership_with_is_enabled() {
+		let rng = &mut test_rng();
+		let root = Fq::rand(rng);
+		let mut set = [Fq::rand(rng); TEST_M];
+		set[0] = root;
+		let not_enabled = Fq::zero();
+		let random_element = Fq::rand(rng);
+
+		let s = TestSetMembership::generate_secrets(&root, &set).unwrap();
+		let is_member =
+			TestSetMembership::check_is_enabled(&random_element, &set, &s, &not_enabled).unwrap();
+
+		assert!(is_member);
+
+		let is_enabled = Fq::rand(rng);
+		let is_member = TestSetMembership::check_is_enabled(&root, &set, &s, &is_enabled).unwrap();
+
+		assert!(is_member);
+	}
+
 	#[test]
 	fn should_not_test_membership() {
 		let rng = &mut test_rng();
@@ -95,4 +134,22 @@ mod test {
 
 		assert!(!is_member);
 	}
+
+	#[test]
+	fn should_not_test_membership_is_enabled() {
+		let rng = &mut test_rng();
+		let root = Fq::rand(rng);
+		let mut set = [Fq::rand(rng); TEST_M];
+		set[0] = root;
+
+		//let not_enabled = Fq::zero();
+		let is_enabled = Fq::rand(rng);
+
+		let s = TestSetMembership::generate_secrets(&root, &set).unwrap();
+		let random_element = Fq::rand(rng);
+		let is_member =
+			TestSetMembership::check_is_enabled(&random_element, &set, &s, &is_enabled).unwrap();
+
+		assert!(!is_member);
+	}
 }
